BC_RollDKP = LibStub("AceAddon-3.0"):NewAddon("BC /RollDKP", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")

local BCRD_VERSION = GetAddOnMetadata("BC_RollDKP", "Version") 

local BCRD_TR = false --TimerRunning Boolean
local BCRD_RIP = false --Roll In Progress Boolean
local BCRD_TH = nil --TimerRoll Handler
local BCRD_ML = "" --MasterLooter Variable

local BCRD_CR = false --TimerCheck Handler
local BCRD_CH = nil --TimerCheck Handler
local BCRD_RR = {} --RaidRoster Table
local BCRD_RS = 0 --RosterSize Variable
local BCRD_NR = 0 --ChecksReceived Variable

function BC_RollDKP:OnInitialize()
    -- Called when the addon is loaded
	self:RegisterChatCommand("bcrd", "ChatCommand")
	self:RegisterChatCommand("bcrd help", "ChatCommand")
	self:RegisterChatCommand("bc_rolldkp", "ChatCommand")
	self:RegisterChatCommand("bcrolldkp", "ChatCommand")
	self:RegisterChatCommand("rolldkp", "RollDKP")
	self:RegisterChatCommand("dkproll", "Fail")
	self:RegisterChatCommand("bcrdver", "GetVersion")
	self:RegisterChatCommand("bcrdvercheck", "StartCheckVer")
	self:RegisterChatCommand("bcrdcheckver", "StartCheckVer")
	self:RegisterChatCommand("bcrdvc", "StartCheckVer")
	self:RegisterChatCommand("bcrdcv", "StartCheckVer")
    self:Print("Version " .. BCRD_VERSION .. " initialized. Type /bcrd for instructions.")
end

function BC_RollDKP:OnEnable()
    -- Called when the addon is enabled
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("CHAT_MSG_ADDON")
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER", self.filterFunc)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", self.filterFunc)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_AFK", self.filterFunc)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_DND", self.filterFunc)
end

function BC_RollDKP:OnDisable()
    -- Called when the addon is disabled
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER", self.filterFunc)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_WHISPER_INFORM", self.filterFunc)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_AFK", self.filterFunc)
	ChatFrame_RemoveMessageEventFilter("CHAT_MSG_DND", self.filterFunc)
end

function BC_RollDKP:GetVersion()
	self:Print("Version " .. BCRD_VERSION)
end

function BC_RollDKP:StartCheckVer()
	if(IsRaidOfficer()) then
		if(not BCRD_CR) then
			BCRD_RR = {} 
			BCRD_RS = 0
			BCRD_NR = 0
			for i=1,40 do 
				local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(i);
				if(name ~= nil) then
					if(online) then
						BCRD_RR[name] = "???"
						BCRD_RS = BCRD_RS + 1
					else
						BCRD_RR[name] = "Offline"
					end
				end
			end
			self:Print("Starting Version Check...")
			self:CancelTimer(BCRD_CH,true)
			BCRD_CH = self:ScheduleTimer("EndCheckVer", 5)
			BCRD_CR = true
			SendAddonMessage("BCRD", "CheckVer?", "RAID")
		else
			self:Print("A version check is already in progress.")
		end
	else
		self:Print("You must be a raid assistant to perform this action.")
	end
end

function BC_RollDKP:EndCheckVer()
	self:CancelTimer(BCRD_CH,true)
	BCRD_CR = false
	local VC_RARR = {}
	for index,value in pairs(BCRD_RR) do
		if(VC_RARR[value] == nil) then
			VC_RARR[value] = {}
		end
		table.insert(VC_RARR[value],index)
		table.sort(VC_RARR[value])
	end
	local VC_VERSIONS = {}
	for index in pairs(VC_RARR) do
		table.insert(VC_VERSIONS,index)
	end
	table.sort(VC_VERSIONS,function(a,b) return a<b end)
	for index,value in pairs(VC_VERSIONS) do
		if((value ~= "???") and (value ~= "Offline")) then
			self:Print("v" .. value .. " -- " .. table.concat(VC_RARR[value],", "))
		end
	end
	if(VC_RARR["???"] ~= nil) then
		self:Print("Old/None -- " .. table.concat(VC_RARR["???"],", "))
	end
	if(VC_RARR["Offline"] ~= nil) then
		self:Print("Offline -- " .. table.concat(VC_RARR["Offline"],", "))
	end
end

function BC_RollDKP:ChatCommand()
	self:Print("Instructions:\n    1. Join a Party or Raid with a Master Looter.\n    2. Wait for something you want to spend DKP on.\n    3. Type /rolldkp instead of normal /roll when you want to use your DKP!\n    (Your DKP will be added to your roll.)\nOther commands:\n    /bcrdver - Displays your current version.\n    /bcrdvercheck - Check the versions of players in your raid (must be raid assistant).")
end

function BC_RollDKP:Fail()
	self:Print("l2spell. It's /rolldkp")
end

function BC_RollDKP:RollDKP()
	if (BCRD_RIP) then
		self:Print("Roll in progress. Wait 5 seconds and try again.")
		return
	end
	lootmethod, masterlooterPartyID, masterlooterRaidID = GetLootMethod()
	if (lootmethod == "master") then
		self:StartRoll()
		self:Print("Just let me find out how much DKP you have...")
		if(masterlooterPartyID) then
			if (masterlooterPartyID == 0) then
				BCRD_ML = UnitName("player")
			else
				BCRD_ML = UnitName("party"..masterlooterPartyID)
			end
			SendChatMessage("!dkp", "WHISPER", nil, BCRD_ML)
		elseif(masterlooterRaidID) then
			BCRD_ML = UnitName("raid"..masterlooterRaidID)
			SendChatMessage("!dkp", "WHISPER", nil, BCRD_ML)
		end
	else
		self:Print("How are you gonna spend your DKP if there's no Master Looter?!")
	end	
end

function BC_RollDKP:CHAT_MSG_WHISPER(info, msg,author,lang)
	if (BCRD_TR) then
		if(author == BCRD_ML) then
			local strMin, strMax = string.find(msg,"WebDKP: Current DKP - ")
			if strMax then
				BCRD_RIP = false
				local dkp = string.sub(msg, (strMax+3))
				local dkpnum = tonumber(dkp)
				if (dkpnum > 0) then
					local genderTable = { "its", "his", "her" }
					SendChatMessage("BC /RollDKP: " .. UnitName("player") .. " is using " .. genderTable[UnitSex("player")] .. " " .. dkp .. " DKP on this roll.", "WHISPER", nil, BCRD_ML)
					self:Print("...got it! Rolling with " .. dkp .. " DKP...")
					RandomRoll(1+dkpnum, 100+dkpnum)
				else
					self:Print("...got it! You currently have " .. dkp .. " DKP, so I'll roll 1-100 instead...")
					RandomRoll(1, 100)
				end
				return
			end
			local strMin, strMax = string.find(msg,"You have no DKP history")
			if strMax then
				self:KillRoll()
				self:Print("You're not currently in the DKP database. This may be a problem, or you may have not earned any DKP yet.")
				return
			end
		end
	end

end

function BC_RollDKP:PARTY_LOOT_METHOD_CHANGED()
	if(BCRD_RIP) then
		self:KillRoll()
		self:Print("Hang on, something changed with the Loot rules. Try again.")
	end
end

function BC_RollDKP:PARTY_MEMBERS_CHANGED()
	if( BCRD_RIP and (GetNumPartyMembers() == 0)and(GetNumRaidMembers() == 0)) then
		self:KillRoll()
		self:Print("Uh... where'd everyone go?")
	end
end

function BC_RollDKP:StartRoll()
	self:CancelTimer(BCRD_TH,true)
	BCRD_TH = self:ScheduleTimer("TimersUp", 5)
	BCRD_TR = true
	BCRD_RIP = true
	BCRD_ML = ""
end

function BC_RollDKP:TimersUp()
	BCRD_TR = false
	BCRD_ML = ""
	if (BCRD_RIP) then
		self:Print("Well *that* didn't work. Could be lag, a disconnect, or maybe your Master Looter doesn't have WebDKP.")
		BCRD_RIP = false
	end
end

function BC_RollDKP:KillRoll()
	self:CancelTimer(BCRD_TH,true)
	BCRD_TR = false
	BCRD_RIP = false
	BCRD_ML = ""
end

function BC_RollDKP.filterFunc(ChatFrameSelf, event, msg, arg2, ...)
	--print("event-"..event.." | msg-" .. msg .. " | arg2-" .. arg2 )
	if (BCRD_TR) then
	    if ( event and msg and arg2 ) then
	        -- whisper too me
	        if ( (event == "CHAT_MSG_WHISPER") and (arg2 == BCRD_ML) and string.find(msg,"^WebDKP: " ) ) then
	                return msg, arg2, ...
	        end
	        -- whisper I am sending to ml
	        if ( (event == "CHAT_MSG_WHISPER_INFORM") and (arg2 == BCRD_ML) and 
			  ((string.find(string.lower(msg),"!dkp")==1) or string.find(msg,"^BC /RollDKP: ")) )  then
	            -- don't display whispercast whisper
	            return msg, arg2, ...
	        end
			-- afk from ml
			if ( ((event == "CHAT_MSG_AFK") or (event == "CHAT_MSG_DND")) and (arg2 == BCRD_ML)) then
				return msg, arg2, ...
			end
	    end
	end
end

function BC_RollDKP:CHAT_MSG_ADDON(event, prefix, message, channel, sender)
	if((prefix == "BCRD") and (channel == "RAID") and (message == "CheckVer?")) then
		SendAddonMessage("BCRD","CheckVer!" .. sender .. "!" .. BCRD_VERSION, "RAID")
	end
	if((BCRD_CR) and (prefix == "BCRD") and (channel == "RAID") and string.find(message, "^CheckVer!" .. UnitName("player") .."!"))then
		local strMin, strMax = string.find(message,"^CheckVer!" .. UnitName("player").."!")
		BCRD_RR[sender] = string.sub(message,strMax+1)
		BCRD_NR = BCRD_NR + 1
		if(BCRD_NR == BCRD_RS) then
			self:EndCheckVer()
		end
	end
end